/*
** ADM-XRC Series Linux Driver
**
** (C) Copyright 2001-2004 Alpha Data
**
** linux_params.c - Declare the module parameters.
**
** All of the recognised module parameters must be declared here, so that they
** can be picked up as driver parameters by the generic section of the driver.
**
** ----------------------------------------------------------------------------
*/

#include "platform.h"
#include "device.h"

static long PCIReadMultiple = 1;
static long PCIWriteAndInvalidate = 1;
static long PCI9656ErrataFix = 1;
static long PCI9656DmaDisable = 0;
static long PCI9656RegsAsSpace = 0;
static long MaxDmaDesc = ADMXRC_MAX_DMADESC;
static long MaxPlxDesc = ADMXRC_MAX_PLXDESC_PER_CHANNEL;
static long CommonBufferSize = 0;
static long ICD2061PMax = 131;
static long ADMXRCMaxLCLK = 40001000;
static long ADMXRCMinLCLK = 399000;
static long ADMXRCPMaxLCLK = 40001000;
static long ADMXRCPMinLCLK = 399000;
static long ADMXRC2LMaxLCLK = 40001000;
static long ADMXRC2LMinLCLK = 399000;
static long ADMXRC2MaxLCLK = 66001000;
static long ADMXRC2MinLCLK = 399000;
static long ADMXRC2PLMaxLCLK = 80001000;
static long ADMXRC2PLMinLCLK = 5999000;
static long ADMXRC2PLUnlockFlash = 0;
static long ADPWRC2MaxLCLK = 66001000;
static long ADPWRC2MinLCLK = 399000;
static long ADPDRC2MaxLCLK = 66001000;
static long ADPDRC2MinLCLK = 399000;
static long ADMXRC2PMaxLCLK = 80001000;
static long ADMXRC2PMinLCLK = 5999000;
static long ADMXRC2PUnlockFlash = 0;
static long ADPXPIMaxLCLK = 80001000;
static long ADPXPIMinLCLK = 5999000;
static long ADPXPIUnlockFlash = 0;
static long ADMXRC4LSMaxLCLK = 66001000;
static long ADMXRC4LSMinLCLK = 399000;
static long ADMXRC4LSSmap32Bit = 0;
static long ADMXRC4LXMaxLCLK = 66001000;
static long ADMXRC4LXMinLCLK = 399000;
static long ADMXRC4LXSmap32Bit = 0;
static long ADMXRC4SXMaxLCLK = 66001000;
static long ADMXRC4SXMinLCLK = 399000;
static long ADMXRC4SXSmap32Bit = 0;

static struct {
  char*           mnemonic;
  DriverParamType type;
  void*           value;
} module_params[] = {
  { "PCIReadMultiple", DriverParamType_boolean, &PCIReadMultiple },
  { "PCIWriteAndInvalidate", DriverParamType_boolean, &PCIWriteAndInvalidate },
  { "PCI9656DmaDisable", DriverParamType_boolean, &PCI9656DmaDisable },
  { "PCI9656ErrataFix", DriverParamType_boolean, &PCI9656ErrataFix },
  { "PCI9656RegsAsSpace", DriverParamType_boolean, &PCI9656RegsAsSpace },
  { "MaxDmaDesc", DriverParamType_uint32, &MaxDmaDesc },
  { "MaxPlxDesc", DriverParamType_uint32, &MaxPlxDesc },
  { "CommonBufferSize", DriverParamType_uint32, &CommonBufferSize },
  { "ICD2061PMax", DriverParamType_uint32, &ICD2061PMax },
  { "ADMXRCMaxLCLK", DriverParamType_uint32, &ADMXRCMaxLCLK },
  { "ADMXRCMinLCLK", DriverParamType_uint32, &ADMXRCMinLCLK },
  { "ADMXRCPMaxLCLK", DriverParamType_uint32, &ADMXRCPMaxLCLK },
  { "ADMXRCPMinLCLK", DriverParamType_uint32, &ADMXRCPMinLCLK },
  { "ADMXRC2LMaxLCLK", DriverParamType_uint32, &ADMXRC2LMaxLCLK },
  { "ADMXRC2LMinLCLK", DriverParamType_uint32, &ADMXRC2LMinLCLK },
  { "ADMXRC2MaxLCLK", DriverParamType_uint32, &ADMXRC2MaxLCLK },
  { "ADMXRC2MinLCLK", DriverParamType_uint32, &ADMXRC2MinLCLK },
  { "ADMXRC2PLMaxLCLK", DriverParamType_uint32, &ADMXRC2PLMaxLCLK },
  { "ADMXRC2PLMinLCLK", DriverParamType_uint32, &ADMXRC2PLMinLCLK },
  { "ADMXRC2PLUnlockFlash", DriverParamType_boolean, &ADMXRC2PLUnlockFlash },
  { "ADPWRC2MaxLCLK", DriverParamType_uint32, &ADPWRC2MaxLCLK },
  { "ADPWRC2MinLCLK", DriverParamType_uint32, &ADPWRC2MinLCLK },
  { "ADPDRC2MaxLCLK", DriverParamType_uint32, &ADPDRC2MaxLCLK },
  { "ADPDRC2MinLCLK", DriverParamType_uint32, &ADPDRC2MinLCLK },
  { "ADMXRC2PMaxLCLK", DriverParamType_uint32, &ADMXRC2PMaxLCLK },
  { "ADMXRC2PMinLCLK", DriverParamType_uint32, &ADMXRC2PMinLCLK },
  { "ADMXRC2PUnlockFlash", DriverParamType_boolean, &ADMXRC2PUnlockFlash },
  { "ADPXPIMaxLCLK", DriverParamType_uint32, &ADPXPIMaxLCLK },
  { "ADPXPIMinLCLK", DriverParamType_uint32, &ADPXPIMinLCLK },
  { "ADPXPIUnlockFlash", DriverParamType_boolean, &ADPXPIUnlockFlash },
  { "ADMXRC4LSMaxLCLK", DriverParamType_uint32, &ADMXRC4LSMaxLCLK },
  { "ADMXRC4LSMinLCLK", DriverParamType_uint32, &ADMXRC4LSMinLCLK },
  { "ADMXRC4LSSmap32Bit", DriverParamType_boolean, &ADMXRC4LSSmap32Bit },
  { "ADMXRC4LXMaxLCLK", DriverParamType_uint32, &ADMXRC4LXMaxLCLK },
  { "ADMXRC4LXMinLCLK", DriverParamType_uint32, &ADMXRC4LXMinLCLK },
  { "ADMXRC4LXSmap32Bit", DriverParamType_boolean, &ADMXRC4LXSmap32Bit },
  { "ADMXRC4SXMaxLCLK", DriverParamType_uint32, &ADMXRC4SXMaxLCLK },
  { "ADMXRC4SXMinLCLK", DriverParamType_uint32, &ADMXRC4SXMinLCLK },
  { "ADMXRC4SXSmap32Bit", DriverParamType_boolean, &ADMXRC4SXSmap32Bit },
  { NULL } /* Terminator */
};

MODULE_LICENSE("GPL");

MODULE_PARM(PCIReadMultiple, "l");
MODULE_PARM(PCIWriteAndInvalidate, "l");
MODULE_PARM(PCI9656DmaDisable, "l");
MODULE_PARM(PCI9656ErrataFix, "l");
MODULE_PARM(PCI9656RegsAsSpace, "l");
MODULE_PARM(MaxDmaDesc, "l");
MODULE_PARM(MaxPlxDesc, "l");
MODULE_PARM(CommonBufferSize, "l");
MODULE_PARM(ICD2061PMax, "l");
MODULE_PARM(ADMXRCMaxLCLK, "l");
MODULE_PARM(ADMXRCMinLCLK, "l");
MODULE_PARM(ADMXRCPMaxLCLK, "l");
MODULE_PARM(ADMXRCPMinLCLK, "l");
MODULE_PARM(ADMXRC2LMaxLCLK, "l");
MODULE_PARM(ADMXRC2LMinLCLK, "l");
MODULE_PARM(ADMXRC2MaxLCLK, "l");
MODULE_PARM(ADMXRC2MinLCLK, "l");
MODULE_PARM(ADMXRC2PLMaxLCLK, "l");
MODULE_PARM(ADMXRC2PLMinLCLK, "l");
MODULE_PARM(ADMXRC2PLUnlockFlash, "l");
MODULE_PARM(ADPWRC2MaxLCLK, "l");
MODULE_PARM(ADPWRC2MinLCLK, "l");
MODULE_PARM(ADPDRC2MaxLCLK, "l");
MODULE_PARM(ADPDRC2MinLCLK, "l");
MODULE_PARM(ADMXRC2PMaxLCLK, "l");
MODULE_PARM(ADMXRC2PMinLCLK, "l");
MODULE_PARM(ADMXRC2PUnlockFlash, "l");
MODULE_PARM(ADPXPIMaxLCLK, "l");
MODULE_PARM(ADPXPIMinLCLK, "l");
MODULE_PARM(ADPXPIUnlockFlash, "l");
MODULE_PARM(ADMXRC4LSMaxLCLK, "l");
MODULE_PARM(ADMXRC4LSMinLCLK, "l");
MODULE_PARM(ADMXRC4LSSmap32Bit, "l");
MODULE_PARM(ADMXRC4LXMaxLCLK, "l");
MODULE_PARM(ADMXRC4LXMinLCLK, "l");
MODULE_PARM(ADMXRC4LXSmap32Bit, "l");
MODULE_PARM(ADMXRC4SXMaxLCLK, "l");
MODULE_PARM(ADMXRC4SXMinLCLK, "l");
MODULE_PARM(ADMXRC4SXSmap32Bit, "l");

static int
charlower(int c)
{
  if (c >= 'A' && c <= 'Z') {
    return c - 'A' + 'a';
  } else {
    return c;
  }
}

static os_boolean
strNoCaseCmp(
  const char* s1,
  const char* s2)
{
  char a, b;

  while (1) {
    a = charlower(*s1++);
    b = charlower(*s2++);
    if (a != b) {
      return FALSE;
    } else {
      if (a == '\0') {
	break;
      }
    }
  }
  return TRUE;
}

/*
** -----------------------------------------------------------------------
** Exported routines
** -----------------------------------------------------------------------
*/

os_boolean
os_get_driver_param(
  OS_DRIVER_OBJECT* pDrvObj,
  const char*       mnemonic,
  DriverParamType   type,
  void*             value,
  unsigned long     size)
{
  unsigned int i;

  for (i = 0; module_params[i].mnemonic != NULL; i++) {
    if (strNoCaseCmp(module_params[i].mnemonic, mnemonic) && module_params[i].type == type) {
      switch (type) {
      case DriverParamType_uint32:
	*((os_uint32*) value) = (os_uint32) *((unsigned long*) module_params[i].value);
	break;
	
      case DriverParamType_int32:
	*((os_int32*) value) = (os_int32) *((long*) module_params[i].value);
	break;
	
      case DriverParamType_boolean:
	*((os_boolean*) value) = (os_boolean) *((unsigned long*) module_params[i].value);
	break;
	
      default:
	break;
      }
      break;
    }
  }

  return FALSE;
}
